
#include "gt.h"
#include "inetraw.h"

/* these 3 are for handling libpcap correctly */
S rawifconf *pcap_ifconfp;
void pcap_alarm (int sigio);

#define ifip_addr(a) (((SAin *)&(a).ifr_addr)->sin_addr)

/* matches a target ip address to the proper network interface */
int
addr2ifname (char *ifname, S in_addr ipaddr)
{
   char eb[PCAP_ERRBUF_SIZE];
   strcpy (ifname, pcap_lookupdev (eb));
   return (!ifname);
/*   int sd, i;
   S ifconf ifc;
   S ifreq ifr[IFR_MAX];
   S in_addr net_id[2], netmask;

   ifc.ifc_len = SO(ifr);
   ifc.ifc_buf = (char *)ifr;

   if((sd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
   perror("socket()");
   return(-1);
   }

   if(ioctl(sd, SIOCGIFCONF, &ifc)) {
   perror("ioctl()");
   return(-1);
   }

   for(i = 0; i < ifc.ifc_len / SO(S ifreq); i++) {
   if(ioctl(sd, SIOCGIFNETMASK, &ifr[i])) {
   perror("ioctl()");
   return(-1);
   }

   netmask.s_addr = ifip_addr(ifr[i]).s_addr;

   if(ioctl(sd, SIOCGIFADDR, &ifr[i])) {
   perror("ioctl()");
   return(-1);
   }

   net_id[0].s_addr = ipaddr.s_addr & netmask.s_addr;
   net_id[1].s_addr = ifip_addr(ifr[i]).s_addr & netmask.s_addr;

   printf("%s : ", inet_ntoa(net_id[0]));
   printf("%s\n", inet_ntoa(net_id[1]));

   if(net_id[0].s_addr == net_id[1].s_addr) {
   strcpy(ifname, ifr[i].ifr_ifrn.ifrn_name);
   close(sd);
   return(0);
   }
   }

   close(sd);
   return(-1);
 */
}

/* converts a network interface name to its ip address */
int
ifname2addr (S in_addr * ipaddr, char *ifname)
{
   int sd;
   S ifreq iff;

   strcpy (iff.ifr_name, ifname);

   if ((sd = socket (AF_INET, SOCK_DGRAM, 0)) == -1) {
      perror ("socket()");
      return (-1);
   }
   if (ioctl (sd, SIOCGIFADDR, &iff)) {
      perror ("ioctl()");
      close (sd);
      return (-1);
   }
   ipaddr->s_addr = ifip_addr (iff).s_addr;
   close (sd);
   return (0);
}

/* initialize descriptors for the proper interface :
 * the only value of the rawifconf structure needed to be filled in prior
 * to a call to init_packet_devices() is the target ip address of ifconfp
 */
int
init_packet_devices (S rawifconf * ifconfp)
{
   char eb[PCAP_ERRBUF_SIZE];
   int on = 1;

   if (!*ifconfp->ri_if) {
      if (addr2ifname (ifconfp->ri_if, ifconfp->ri_dst)) {
	 printf ("addr2ifname(): network unreachable\n");
	 return (-1);
      }
   }
   if (ifname2addr (&ifconfp->ri_src, ifconfp->ri_if)) {
      printf ("ifname2addr(): unknown interface\n");
      return (-1);
   }
   ifconfp->ri_pd = pcap_open_live (ifconfp->ri_if, PMIN * 4, 0, PMIN, eb);
   if (!ifconfp->ri_pd) {
      perror ("pcap_open_live()");
      return (-1);
   }
   switch (pcap_datalink (ifconfp->ri_pd)) {
   case DLT_NULL:		/* no link-layer encapsulation */
      ifconfp->ri_off = 4;
      break;
   case DLT_EN10MB:		/* Ethernet (10Mb) */
      ifconfp->ri_off = 14;
      break;
   case DLT_SLIP:		/* Serial Line IP */
   case DLT_PPP:		/* Point-to-point Protocol */
      ifconfp->ri_off = 24;
      break;
   case DLT_EN3MB:		/* Experimental Ethernet (3Mb) */
   case DLT_AX25:		/* Amateur Radio AX.25 */
   case DLT_PRONET:		/* Proteon ProNET Token Ring */
   case DLT_CHAOS:		/* Chaos */
   case DLT_IEEE802:		/* IEEE 802 Networks */
   case DLT_ARCNET:		/* ARCNET */
   case DLT_FDDI:		/* FDDI */
   case DLT_ATM_RFC1483:	/* LLC/SNAP encapsulated atm */
   case DLT_RAW:		/* raw IP */
   case DLT_SLIP_BSDOS:	/* BSD/OS Serial Line IP */
   case DLT_PPP_BSDOS:		/* BSD/OS Point-to-point Protocol */
      printf ("unsupported datalink type\n");
      return (-1);
      break;
   default:
      printf ("unknown datalink type\n");
      return (-1);
      break;
   }
   ifconfp->ri_sd = socket (AF_INET, SOCK_RAW, IPPROTO_ICMP);
   if (ifconfp->ri_sd == -1) {
      pcap_close (ifconfp->ri_pd);
      perror ("socket()");
      return (-1);
   }
   if (setsockopt (ifconfp->ri_sd, IPPROTO_IP, IP_HDRINCL, &on, SO (on)) ==
       -1) {
      printf ("setsockopt()\n");
      return (-1);
   }
   return (0);
}

void
term_packet_devices (S rawifconf * ifconfp)
{
   close (ifconfp->ri_sd);
   pcap_close (ifconfp->ri_pd);
}

void
pcap_alarm (int sigio)
{
   sendicmp (pcap_ifconfp, ICMP_ECHO, 0, 0, 128);

   alarm (1);			/* nasty way to restart the alarm timer */
}

int
get_packet (S rawifconf * ifconfp, S ip * ipp,
	    char *packet, int pkt_len, int ck_side)
{
   S ip ip;
   S pcap_pkthdr pcap_hdr;
   char *p = NULL;
   int iphl;
   u_short sum, oldsum;

   if (!ipp)
      ipp = &ip;

   bzero (packet, pkt_len);
   bzero (ipp, SO (ip));

   signal (SIGALRM, pcap_alarm);
   pcap_ifconfp = ifconfp;

   alarm (1);
   if (!(p = (char *) pcap_next (ifconfp->ri_pd, &pcap_hdr)))
      return (-1);
   alarm (0);

   p += ifconfp->ri_off;

   memcpy (ipp, p, min (SO (ip), pcap_hdr.caplen));

   if (ipp->ip_v != IPVERSION || ipp->ip_hl > 5)
      return (-1);

   iphl = ipp->ip_hl * 4;

   oldsum = ipp->ip_sum;
   ipp->ip_sum = 0;
   sum = in_cksum ((u_short *) ipp, iphl);
   ipp->ip_sum = oldsum;

   if (sum == oldsum)
      if (ck_side == CHK_SOURCE_IP ?
	  ipp->ip_src.s_addr : ipp->ip_dst.s_addr == ifconfp->ri_dst.s_addr) {

	 memcpy (packet, p + iphl, min (pkt_len, pcap_hdr.caplen));

	 return (0);
      }
   return (-1);
}

int
sendicmp (S rawifconf * ifconfp, u_char type, u_char code, u_short seq,
	  u_char ttl)
{
   int i;
   SAin sa_d;
   S {
      S ip ip;
      S icmp icmp;
   }
   p;
   bzero (&p, SO (p));

   p.ip.ip_v = 4;
   p.ip.ip_hl = SO (S ip) >> 2;
   p.ip.ip_len = htons (SO (p));
   p.ip.ip_id = getpid ();
   p.ip.ip_ttl = ttl;
   p.ip.ip_p = IPPROTO_ICMP;
   p.ip.ip_src.s_addr = ifconfp->ri_src.s_addr;
   p.ip.ip_dst.s_addr = ifconfp->ri_dst.s_addr;

   p.icmp.icmp_type = type;
   p.icmp.icmp_code = code;
   p.icmp.icmp_id = getpid ();
   p.icmp.icmp_seq = seq;

   p.icmp.icmp_cksum = in_cksum ((u_short *) & p.icmp, SO (S icmp));
   p.ip.ip_sum = in_cksum ((u_short *) & p.ip, SO (S ip));

   sa_d.sin_family = AF_INET;
   sa_d.sin_addr = ifconfp->ri_dst;

   for (i = 5; i; i--) {
      if (!sendto (ifconfp->ri_sd, &p, SO (p), 0, (SA *) & sa_d, SO (sa_d)) !=
	  SO (p))
	 return (0);
      sleep (1);
   }
   return (-1);
}
