
#include "gt.h"

/* general inet wrapper functions and other numerical things */

void
generic_sigalrm (int sigio)
{
}

char *
ipv4_showname (S in_addr ipaddr)
{
#define HOSTNAME_LEN 40
   static char hostname[HOSTNAME_LEN];
   S hostent *hentry;

   if ((hentry = gethostbyaddr ((char *) &ipaddr, 4, AF_INET))) {
      if (strlen (hentry->h_name) < HOSTNAME_LEN) {
	 strcpy (hostname, hentry->h_name);
	 return (hostname);
      }
   }
   return (strcpy (hostname, inet_ntoa (ipaddr)));
}

int
ipv4_rslvhost (char *hostname, S in_addr * ipaddrp)
{
   S hostent *hentry;

   if (!inet_aton (hostname, ipaddrp)) {
      if ((hentry = gethostbyname (hostname)))
	 memcpy (ipaddrp, hentry->h_addr, hentry->h_length);
      else
	 return (0);
   }
   return (-1);
}

u_short
in_cksum (u_short * addr, int len)
{
   int nleft = len;
   int sum = 0;

   u_short *w = addr;
   u_short answer = 0;

   /* 
    * Our algorithm is simple, using a 32 bit accumulator (sum), we add
    * sequential 16 bit words to it, and at the end, fold back all the
    * carry bits from the top 16 bits into the lower 16 bits.
    */

   while (nleft > 1) {
      sum += *w++;
      nleft -= 2;
   }

   /* mop up an odd byte, if necessary */
   if (nleft == 1) {
      *(unsigned char *) (&answer) = *(unsigned char *) w;
      sum += answer;
   }
   /* 4add back carry outs from top 16 bits to low 16 bits */
   sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
   sum += (sum >> 16);		/* add carry */
   answer = ~sum;		/* truncate to 16 bits */
   return (answer);
}

u_short
tcp_cksum (char *packet, int len, S in_addr src_addr, S in_addr dst_addr)
{
   char *temppkt;
   u_short cksum;

   S temphdr
   {
      S in_addr src;
      S in_addr dst;
      unsigned char zero;
      unsigned char pr;
      u_short len;
   }
   temphdr;

   temphdr.pr = IPPROTO_TCP;
   temphdr.len = htons (len);
   temphdr.zero = 0;

   temphdr.src = src_addr;
   temphdr.dst = dst_addr;

   if ((temppkt = malloc (SO (temphdr) + len)) == NULL) {
      perror ("cannot allocate memory\n");
      exit (EXIT_FAILURE);
   }
   memcpy (temppkt, &temphdr, SO (S temphdr));
   memcpy ((temppkt + SO (temphdr)), packet, len);

   cksum = in_cksum ((unsigned short *) temppkt, (len + SO (temphdr)));
   free (temppkt);
   return cksum;
}

int
hamming (u_long mask)
{
   u_short i, j;
   for (i = j = 0; i < 32; i++)
      if (mask & (1 << i))
	 j++;
   return (j >> 1);
}
